@@ -6,6 +6,7 @@ from com.sun.star.beans import PropertyValue
 
 MAX_MOVES = 32767
 
+
 class oodocument:
     def __init__(self, orig, host="0.0.0.0", port="8001"):
         self.orig = orig
@@ -13,6 +14,7 @@ class oodocument:
         self.port = port
         self.font_color = None
         self.font_back_color = None
+        self.clear_hyperlinks = True
         self.__set_document()
 
     def __str__(self):
@@ -70,10 +72,26 @@ class oodocument:
         else:
             self.convert_to(dest, format)
 
-    def replace_with_index_in_header(self, data=[], dest=None, format=None, offset=0):
+    def replace_with_index_in_header(
+        self,
+        data=[],
+        dest=None,
+        format=None,
+        offset=0,
+        word_neighbors=0,
+        style_name="Default Style",
+    ):
         data.sort(key=lambda x: x[0], reverse=True)
+        try:
+            paragraph = self.document.getStyleFamilies().getByName("PageStyles").getByName(style_name).HeaderText
+        except Exception as e:
+            logging.error(traceback.format_exc())
+            raise
+
         for start_index, end_index, replace, word_check in data:
-            self.__find_and_replace_index(self.document, start_index, end_index, replace, offset)
+            self.__find_and_replace_index(
+                self.document, start_index, end_index, replace, word_check, offset, word_neighbors, paragraph
+            )
         if dest is None and format is None:
             self.save()
         else:
@@ -97,9 +115,15 @@ class oodocument:
         found = document.findFirst(search)
 
         while found:
+            has_hyperlink_props = True if found.getPropertyValue("HyperLinkURL") or found.getPropertyValue("HyperLinkName") or found.getPropertyValue("HyperLinkTarget") else False
             found.String = found.String.replace(find, replace)
             found.setPropertyValue("CharColor", self.font_color) if self.font_color else ""
             found.setPropertyValue("CharBackColor", self.font_back_color) if self.font_back_color else ""
+            if self.clear_hyperlinks and has_hyperlink_props:
+                found.setPropertyValue("HyperLinkURL", "")
+                found.setPropertyValue("HyperLinkName", "")
+                found.setPropertyValue("HyperLinkTarget", "")
+
             found = document.findNext(found.End, search)
 
     def safe_goRight(self, cursor, count, expand):
@@ -121,24 +145,29 @@ class oodocument:
         word_check=None,
         offset=0,
         character_neighbors=20,
+        paragraph_to_replace=None,
     ):
         """
         Searches the given word and replaces it's value with the given
         replacement string.
         """
+        if paragraph_to_replace:
+            text = paragraph_to_replace.Text
+        else:
+            text = document.Text
 
-        text = document.Text
         cursor = text.createTextCursor()
         total_lenght = (end_index + offset) - ((start_index) + offset) - 1
 
         self.safe_goRight(cursor, start_index + offset, False)
         cursor.goRight(total_lenght, True)
         character_word = 0
+
         if word_check != cursor.String:
             character_word = 1
             while word_check != cursor.String and character_neighbors > character_word:
                 cursor.gotoStart(False)
-                self.safe_goRight(cursor, start_index - character_word, False)
+                self.safe_goRight(cursor, start_index + offset - character_word, False)
                 cursor.goRight(total_lenght, True)
                 character_word += 1
 
@@ -151,11 +180,18 @@ class oodocument:
                 character_word += 1
 
         if word_check == cursor.String:
+            has_hyperlink_props = True if cursor.getPropertyValue("HyperLinkURL") or cursor.getPropertyValue("HyperLinkName") or cursor.getPropertyValue("HyperLinkTarget") else False
             cursor.String = replace
             cursor.setPropertyValue("CharColor", self.font_color) if self.font_color else ""
             cursor.setPropertyValue("CharBackColor", self.font_back_color) if self.font_back_color else ""
+            if self.clear_hyperlinks and has_hyperlink_props:
+                cursor.setPropertyValue("HyperLinkURL", "")
+                cursor.setPropertyValue("HyperLinkName", "")
+                cursor.setPropertyValue("HyperLinkTarget", "")
+            cursor.gotoStart(False)
 
-        cursor.gotoStart(False)
+    def set_clear_hyperlinks(self, val):
+        self.clear_hyperlinks = True if val else False
 
     def set_font_color(self, r, g, b):
         self.font_color = self.__rgbToOOColor(r, g, b)
